{********************************************
* ViewPort PropCVCapture       v4.16        *
* Capture and stream Video to the Propeller *
* (C) 2009 myDanceBot.com, Inc.             *
*********************************************}{{
Use this Object to capture video into an array which can be streamed to ViewPort
for display.  Other programs can analyze the data and use ViewPort to display
their output.

Features:
-Capture NTSC video up to 240h x 200v at 16 grayscales into 24kB buffer at 30fps
-Supports lower resolution of 120hx100v at 16 grayscales into a 6kB buffer at 30fps
-Continuously stream video, or take one snapshot
-Realtime tracking of a bright blob without using memory at 30fps

Use:
-Supply Propeller with video signal
        To generate the signal, use a "C Cam 2A" available at http://electronics123.com
        To sample the signal, use a "ADC08100" available at digikey.
        Connect the camera's NTSC composite signal to the ADC's input and connect the
        ADC's output to the Propeller's pins.  See constants below for exact pins.
-Include this object in your program:
        Obj video:"PropCVCapture"
-Allocate memory for the VideoFrame:
        LONG VideoFrame[6000]
-Register with ViewPort        
        vp.register(video.start(@videoFrame,vp#HIVIDEO))
-Analyze VideoFrame
        8 pixels are encoded into each long.  Item 0 is top left.
        ViewPort will show the video image and can mark a box if supplied with
        a variable named visionXYWH- where x is top 8 bits, y is next 8, w is next 8,
        h is last 8.  }}
con
  ''change the first two constants to match where your adc input/output are 
  VIDEOSTART=4    ''pins VIDEOSTART..VIDEOSTART+3 should connect to the 4MSB of the ADC
  ADC_CLOCK_PIN=8 ''pin ADC_CLOCK_PIN should connect to your ADC's clock  
  vsyncns=15000
  hsyncns=1500 
  burstns=4000
  datans=55000
  vblankns=415000

  #0,BLOB,HIVIDEO,HIPIC,VIDEO4,VIDEO2,VIDEO1,PIC
  VIDEOPINS=15<< VIDEOSTART ''mask just the 4 video pins
  SYNCPINS=14<< VIDEOSTART  ''for the sync, look at top 2 bits
  ROT0=32-VIDEOSTART        ''the adc's output, 8 consecutive samples
  ROT1=ROT0+4               ''are rotated into one 32bit word.  
  ROT2=ROT1+4   
  ROT3=ROT2+4
  ROT4=ROT3+4
  ROT5=ROT4+4
  ROT6=ROT5+4
  ROT7=ROT6+4
obj
  Freq : "Synth"  
pub start(VideoPtr,mode)
{{start VideoCapture
 processes incoming video according to these modes
 0 process incoming video at 30fps and write result to par
 1 video capture  200x240x4bits=6000longs
 2 take one snapshot 200x240x4bits=6000longs, then stop
 3 video edit -captures 100x120 into top, room for 3 more=6000 longs
 4 video edit -captures 100x120 into top, room for 1 more=3000 longs 
 5 video capture  100x120x4bits=1500longs
 6 take one snapshot 100x120x4bits=1500longs, then stop}}
  Freq.Synth("B",ADC_CLOCK_PIN, 10_000_000)
  word[videoptr]:=mode 
  cognew(@doVideo,VideoPtr)
  return VideoPtr
dat
              org 0
doVideo       rdlong    sA,par
              add       sA,#:jmpTable
              jmp       sA
 :jmpTable    jmp       #ProcessVideo   
              jmp       #HighVideo      
              jmp       #HighSnap       
              jmp       #LowVideo       
              jmp       #LowVideo       
              jmp       #LowVideo       
              jmp       #LowSnap        
ProcessVideo
              call      #findVSync
              mov       fieldCtr,#200              
              mov       brightest,video
:sampleFPic
              call      #findHSync 
              mov       lineCtr,#200
 :sampleLine  mov       sA,ina               
              and       sA,video             
              cmp       sA,brightest wc      
      if_nc   djnz      lineCtr,#:sampleLine 
      if_nc   jmp       #:done               
     
              mov       brightest,sA         
              mov       xB,lineCtr           
              mov       yB,fieldCtr          
:nowr         sub       lineCtr,#2 wz,wc     
              nop                            
              nop                            
if_nz_and_nc  jmp       #:sampleLine         
 :done        djnz      fieldCtr,#:sampleFPic
 :wrback      shl       yB,#16
              add       xB,yB
              wrlong    xB,par 
              jmp       #ProcessVideo
LowVideo      call      #doLoFrame
              jmp       #LowVideo
LowSnap       call      #doLoFrame
              cogid     sA
              cogstop   sA
HighVideo     call      #doHiFrame
              jmp       #HighVideo
HighSnap      call      #doHiFrame
              cogid     sA
              cogstop   sA





doHiFrame     mov       pixPtr,par
              call      #findVSync
              mov       fieldCtr,#200
:sampleField  mov       lineCtr,#30
              call      #findHSync  
:sample8Pix   mov       sA,ina
              and       sA,video
              rol       sA,#ROT0    
              wrlong    sL,pixPtr
              mov       sB,ina
              and       sB,video
              rol       sB,#ROT1
              add       sA,sB       
              mov       sB,ina
              and       sB,video
              rol       sB,#ROT2                            
              mov       sL,sA
              mov       sA,ina
              and       sA,video
              rol       sA,#ROT3
              add       sL,sB       
              add       sL,sA       
              mov       sB,ina
              and       sB,video
              rol       sB,#ROT4
              add       sL,sB       
              mov       sA,ina
              and       sA,video
              rol       sA,#ROT5                              
              add       pixPtr,#4
              mov       sB,ina
              and       sB,video
              rol       sB,#ROT6
              add       sL,sA
              add       sL,sB       
              mov       sA,ina
              and       sA,video
              rol       sA,#ROT7
              add       sL,sA       
              djnz      lineCtr,#:sample8Pix
              djnz      fieldCtr,#:sampleField
              wrlong    sL,pixPtr
doHiFrame_ret ret


doLoFrame     mov       pixPtr,par
              call      #findVSync   
              mov       fieldCtr,#100
:sampleField  mov       lineCtr,#15
              call      #findHSync 


              waitcnt   sB,#0
              call      #findHSync  
:sampleLine   mov       sb,#7     
              mov       sL,#0
:sample8Pix
              mov       sA,ina     
              and       sA,video
              add       sL,sA
              ror       sL,#4
              nop
              nop
              nop
              djnz      sb,#:sample8Pix
              mov       sA,ina
              and       sA,video
              add       sL,sA
              ror       sL,#4+VIDEOSTART
              wrlong    sL,pixptr
              add       pixptr,#4
              djnz      lineCtr,#:sampleLine
              djnz      fieldCtr,#:sampleField
doLoFrame_ret ret

    findVSync
             
              waitpeq   syncval,sync
              mov       sA,cnt 
              waitpne   syncval,sync
              mov       sB,cnt
              sub       sB,sA
              cmp       sB,vSync wc
    if_c      jmp       #findVSync
              mov       sB,cnt
              add       sB,vblank
              waitcnt   sB,cycPline                            
findVSync_ret ret

    findHSync
             
              waitpeq   syncval,sync
              mov       sA,cnt
              waitpne   syncval,sync
              mov       sB,cnt
              sub       sB,sA
              cmp       sB,hSync wc
    if_c      jmp       #findHSync
              mov       sB,cnt
              add       sB,burst
              waitcnt   sB,cycPline
findHSync_ret ret
cycPline      long      4320'-37              
vsync         long      vsyncns*125/1000 
hsync         long      hsyncns*125/1000 
burst         long      burstns*125/1000
vblank        long      vblankns*125/1000  
syncVal       long      0
video         long      VIDEOPINS
sync          long      SYNCPINS
blob55        long      5<<24+5<<16+200<<8+240
ncnt          res       1      
fieldCtr      res       1      
pixPtr        res       1      
lineCtr       res       1      
brightest     res       1
sA            res       1      
sB            res       1
sL            res       1      
xb            res       1
yb            res       1