{********************************************
* ViewPort PropCVFilter              v4.16  *
* Perform Vision algorithms on video data   *
* (C) 2009 myDanceBot.com, Inc.             *
*********************************************}{{
Use this Object to continuously apply vision filters to video data captured
by PropCVCapture.  Spin commands are used to manipulate a vision program which
is run in assembly in its own cog.

Features:
-Implements common vision filters in assembly for real time performance- up to 30fps
-Combine multiple filters using a "vision program"
-Supports 1, 2 or 4 video buffers
-Commands: copy, invert, threshold, difference, chaos, max, pattern

Use:
-Supply Propeller with video signal and capture with PropCVCapture
        See PropCVCapture for details
        Use mode 3 or 4 of PropCVCapture to capture into buffer 0
-Include this object in your program:
        Obj ve:"PropCVFilter"
-Start Vision Object
        ve.start(@videoFrame,video#VIDEO4)
-Program Vision Filters
        ve.filter(0,0,ve#pattern)         
        ve.filter(1,0,ve#invert)          
        ve.filtervalue(2,0,ve#correlate,@b)'store correlate result into b     
-The Vision object will continually apply the programmed filters to incoming video
}}

con
#0,None,Threshold,Invert,Difference,Chaos,Max2,Correlate,Draw,Pattern,Lowest,Copy
var
  long cmd[20]
pub start(tVptr,tmode)
  if tmode>1
    longsNline:=15
    linesNpanel:=100
    bytesNline:=longsNline*4
    longsNpanel:=linesNpanel*longsNline
    bytesNpanel:=longsNpanel*4
  vptr:=tVptr
  cognew(@doEdit,@cmd)
pub filter(tDest,tSrc,tFilter)
  cmd[cmdNum]:=tFilter
  cmd[cmdNum+1]:=vptr+tDest*bytesNpanel
  cmd[cmdNum+2]:=vptr+tSrc*bytesNpanel
  cmdNum+=3  
pub filterValue(tDest,tSrc,tFilter,tVal)
  cmd[cmdNum]:=tFilter
  cmd[cmdNum+1]:=vptr+tDest*bytesNpanel
  cmd[cmdNum+2]:=vptr+tSrc*bytesNpanel
  cmd[cmdNum+3]:=tval
  cmdNum+=4  
dat
' ve.start(@videoFrame)
' ve.copy(0,1)
'cmd=3,vptr,vptr+longsNpanel,0

              org 0
doEdit        mov       cmdPtr,par
cmdLoop       rdlong    action,cmdPtr
              add       cmdPtr,#4
              rdlong    dest,cmdPtr
              add       cmdPtr,#4
              rdlong    src,cmdPtr
              add       cmdPtr,#4
              mov       n,longsNpanel      
              add       action,#:jmpTable
              jmp       action
:jmpTable     jmp       #doEdit
              jmp       #dothreshold
              jmp       #doinvert
              jmp       #dodiffX
              jmp       #dochaos
              jmp       #domax
              jmp       #docorrelate
              jmp       #dodraw
              jmp       #dopattern
              jmp       #dolowest
              jmp       #docopy
dopattern     shr       n,#2
              mov       tmp,n
              mov       new,cnt
:lp1          wrlong    new,dest
              add       dest,#4                     
              djnz      n,#:lp1
              mov       n,tmp
              mov       new,#0
:lp2          wrlong    new,dest
              add       dest,#4                     
              djnz      n,#:lp2
              mov       n,tmp
              mov       new,#$1FF
:lp3          wrlong    new,dest
              add       dest,#4                     
              djnz      n,#:lp3
              mov       n,tmp
              mov       new,#15              
:lp4          wrlong    new,dest
              add       dest,#4                     
              djnz      n,#:lp4                                          
              jmp       #cmdLoop
dodraw        mov       val,#0
              wrlong    val,src
              add       src,#4
              wrlong    val,src                     
              jmp       #cmdLoop                           
docopy        rdlong    old,src
              add       src,#4
              wrlong    old,dest
              add       dest,#4                     
              djnz      n,#docopy
              jmp       #cmdLoop
doChaos
              mov       dnp,src
              add       dnp,bytesNline
              sub       n,#15
              mov       sum,#150
              mov       lt,#0
              mov       t2,#0
:loop         rdlong    old,src
              add       src,#4
              rdlong    dn,dnp
              add       dnp,#4
              mov       m,#8
              mov       new,#0
              
'sum:sum of last 8 differences
't1 is temporary
'tmp 0-3 has new data, lt 0-3 has data from 8 back
't2 has data from 8 back for down
:limit
'jmp       #:a1
              mov       tmp,old
              and       tmp,#15
              add       sum,tmp
              shr       lt,#4 
              mov       t1,lt 
              and       t1,#15
              sub       sum,t1
              shl       tmp,#28
              add       lt,tmp
              
  :a1         mov       tmp,dn
              and       tmp,#15
              add       sum,tmp
              shr       t2,#4 
              mov       t1,t2 
              and       t1,#15
              sub       sum,t1
              shl       tmp,#28
              add       t2,tmp

              mov       t1,sum
              shr       t1,#3
'              mins      t1,#15
              and       t1,#15
              add       new,t1
              ror       new,#4
              ror       old,#4
              ror       dn,#4


              
              djnz      m,#:limit
              
              wrlong    new,dest
              add       dest,#4                     
              djnz      n,#:loop
              jmp       #cmdLoop
                            
doMax



              rdlong    val,cmdPtr
              add       cmdPtr,#4
              mov       dnp,src
              add       dnp,bytesNline
              sub       n,#15
              mov       dest,#0
              mov       sum,#0
              
:loop         rdlong    old,src
              add       src,#4
              rdlong    dn,dnp
              add       dnp,#4
              
              mov       m,# 8
              mov       new,#0
             
             
:dodiffb      mov       tmp,old
              and       tmp,#15
              mov       t1,dn
              and       t1,#15
              add       tmp,t1
              cmp       tmp,sum wc
if_c          jmp       #:notMax
'              mov       dest,m
'              shl       dest,#8
              mov       dest,sum
              shl       dest,#16
              add       dest,n              
              mov       sum,tmp
:notMax       ror       dn,#4
              ror       old,#4
              djnz      m,#:dodiffb
              djnz      n,#:loop
              wrlong    dest,val
              jmp       #cmdLoop              
doDiffX



:loop         rdlong    old,src
              add       src,#4
              mov       m,# 8
              mov       new,#0
             
             
:dodiffb      mov       tmp,old
              and       tmp,#15

              mov       t1,lt  
              sub       t1,tmp 
              abs       t1,t1
              add       new,t1
              mov       lt,tmp

              ror       new,#4
              ror       old,#4
              djnz      m,#:dodiffb
              wrlong    new,dest
              add       dest,#4
              djnz      n,#:loop
              jmp       #cmdLoop              
doDiffXY



              mov       dnp,src
              add       dnp,bytesNline
              sub       n,#15
:loop         rdlong    old,src
              add       src,#4
              rdlong    dn,dnp
              add       dnp,#4
              
              mov       m,# 8
              mov       new,#0
             
             
:dodiffb      mov       tmp,old
              and       tmp,#15
              mov       t1,dn
              and       t1,#15
              sub       t1,tmp
              abs       t1,t1
             

              mov       t1,lt  
              sub       t1,tmp 
              abs       t1,t1
              add       new,t1
              mov       lt,tmp

              ror       new,#4
              ror       dn,#4
              ror       old,#4
              djnz      m,#:dodiffb
              wrlong    new,dest
              add       dest,#4
              djnz      n,#:loop
              jmp       #cmdLoop              
dothreshold   rdlong    val,cmdPtr
              add       cmdPtr,#4
:loop         rdlong    old,src
              add       src,#4
              mov       m,#8
              mov       new,#0
              
:limit        mov       tmp,old
              and       tmp,#15
              cmp       tmp,val wc
      if_nc   add       new,#15
              ror       new,#4
              ror       old,#4
              djnz      m,#:limit
              
              wrlong    new,dest
              add       dest,#4                     
              djnz      n,#:loop
              jmp       #cmdLoop
doLowest
              rdlong    t1,cmdPtr 
              add       cmdPtr,#4  
              mov       t2,#0     
              
              
              mov       n,linesNpanel
 :loopLines   mov       x,longsNline 
              sub       x,#2              
              mov       val,#15      
:loop         rdlong    old,src      
              add       src,#4
              mov       m,#8
              mov       new,#0
              
:limit        mov       tmp,old
              and       tmp,#15
              cmp       tmp,val wc
'      if_c   add       new,#15
'      if_c   mov       val,tmp
if_c          add       new,#15
if_c          sub       val,#1
if_c          mov       t3,x         
              ror       new,#4
              ror       old,#4
              djnz      m,#:limit
              
'              wrlong    new,dest
              add       dest,#4                     
              djnz      x,#:loop
              add       t2,t3
              add       src,#8
              add       dest,#8
              djnz      n,#:loopLines
              wrlong    t2,t1
              jmp       #cmdLoop
                            
doInvert      
:loop         rdlong    old,src
              add       src,#4
              mov       m,#8
              mov       new,#0

:limit        mov       tmp,old
              and       tmp,#15
              neg       tmp,tmp
              add       tmp,#16
              add       new,tmp    
              ror       new,#4
              ror       old,#4
              djnz      m,#:limit
              
              wrlong    new,dest
              add       dest,#4                     
              djnz      n,#:loop
              jmp       #cmdLoop
doCorrelate              
'read 3 longs
              
              rdlong    t1,src
              add       src,#4
              rdlong    t2,src
              add       src,#4
              rdlong    t3,src
              mov       mval,#255
'for shift=0 to 7
              mov       shift,#8        
:comp8x       
'make 8 compares with t1,t2- vs p1,p2
              mov       m,#8
              mov       val,#0
              
:docomp       mov       pix1,t1
              and       pix1,#15
              mov       pix2,p1
              and       pix2,#15  
              sub       pix1,pix2
              addabs    val,pix1  
              ror       t1,#4
              ror       p1,#4
              djnz      m,#:docomp
'see if val is a minimum
              cmp       val,mval wc 
      if_c    mov       mval,val
      if_c    mov       lt,shift                                 
'modify t1,t2 using shift
              mov       m,#4
:doshift      rcr       t3,#1 wc 
              rcr       t2,#1 wc
              rcr       t1,#1
              djnz      m,#:doshift
              djnz      shift,#:comp8x
              
              rdlong    val,cmdPtr
              add       cmdPtr,#4
              shl       mval,#16
              add       mval,lt
              wrlong    mval,val
              jmp       #cmdLoop
              
    longsNline long 0
    linesNpanel long 0                   
bytesNline          long      0     
bytesNpanel          long      0      
longsNpanel           long      0     
vPtr          long      0
cmdNum        long      0
p1            long      $22222222
p2            long      $22222222
sum           res       1
action        res       1
src           res       1
dest          res       1
cmdPtr        res       1
n             res       1
old           res       1
new           res       1
dn            res       1
lt            res       1
dnP           res       1
m             res       1
tmp           res       1
t1            res       1
t2            res       1
t3            res       1
val           res       1
x             res       1
mval          res       1
shift         res       1
pix1          res       1
pix2          res       1